// This is a fairly complicated sketch that just popped into my
// head.  But it has a simple punchline: it's a demonstration
// of writing animated GIFs.  Therefore, you can ignore basically
// all the math that's in here -- that's just a way to get 
// interesting periodic motion.  Focus on the parts related to
// writing GIFs: creating a GifMaker and saving frames.  The
// only tricky bit there is that you have to calculate exactly
// how many frames to save in order to get one complete cycle
// in the resulting GIF.

// Use the GIF library.
import gifAnimation.*;

// Variables to control GIF writing.
GifMaker maker;
int exp_frames;

// The rest of this stuff is all the math to move points around
// in a periodic way.
int num_sides = 8;
int skip = 3;
int num_points = 16;

float inc;
int frames_per_point = 80;

float[] xs;
float[] ys;

void createPolygon()
{
  xs = new float[ num_sides ];
  ys = new float[ num_sides ];
  
  for( int idx = 0; idx < num_sides; ++idx ) {
    float ang = TWO_PI*float(idx)/float(num_sides);
    xs[idx] = width/2 + 160.0*cos( ang );
    ys[idx] = height/2 + 160.0*sin( ang );
  }
}

void calcIncrement()
{
  inc = float(num_sides) / float(num_points) / float(frames_per_point);
}

void setup()
{
  size( 400, 400 );
  smooth(); 
  createPolygon();
  calcIncrement();
  
  // Probably not necessary, but I'll explicitly set the
  // GifMaker to null to make it clear there's no writer
  // at the start of the sketch.
  maker = null;
}

void draw()
{
  // Do a whole bunch of drawing.
  
  background( 240, 230, 220 );

/*
  // This section of code draws a star-shaped polygon 
  // underneath the points.  If you uncomment it, it
  // gives you a much clearer sense of what the points
  // are doing.
  stroke( 0, 100 );
  strokeWeight( 1.0 );
  noFill();
  beginShape();
  for( int idx = 0; idx < num_sides; ++idx ) {
    vertex( xs[idx*skip%num_sides], ys[idx*skip%num_sides] );
  }
  endShape( CLOSE );
*/ 
  
  stroke( 0 );
  strokeWeight( 1.5 );
  fill( 80, 90, 100, 50 );
   
  float phase = float(frameCount) * inc;
  
  for( int idx = 0; idx < num_points; ++idx ) {
    float t = float(idx)/float(num_points) * float(num_sides);
    t = (t + phase) % float(num_sides);
    
    int si = int( t );
    float rem = t - si;
    si = (si*skip) % num_sides;
    int ei = (si+skip)%num_sides;

    // Use easing to smooth point motion (we'll talk about this
    // in the physics section of the course.
    rem = 0.5*(1-cos( PI*rem ));
    
    ellipse(
      lerp( xs[si], xs[ei], rem ), 
      lerp( ys[si], ys[ei], rem ), 
      35, 35 );
  }
  
  // Here's the only relevant bit.  If we're currently writing
  // a GIF, then take the image you just rendered and save it.
  if( maker != null ) {
    // The next frame should be displayed for 16ms, or about
    // 1/60 of a second.
    maker.setDelay( 16 );
    // Add the current window to the GIF.
    maker.addFrame();
    ++exp_frames;
    if( exp_frames >= frames_per_point ) {
      println( "Wrote " + exp_frames + " frames." );
      // We're done.  Clean up.
      maker.finish();
      maker = null;
    }
  }
}

// Look at all the (key == '?') conditions below to
// figure out what keys you can press in this sketch.
// The g key writes out an animated GIF.  The 
// q,w,a,s,z and x keys control the points.  It can
// be fun to experiment with them.
void keyPressed()
{
  if( key == 'q' ) {
    // Don't go below two sides.
    num_sides = max( 2, num_sides - 1 );
    createPolygon();
    calcIncrement();
  } else if( key == 'w' ) {
    ++num_sides;
    createPolygon();
    calcIncrement();
  } else if( key == 'a' ) {
    skip = max( 1, skip - 1 );
  } else if( key == 's' ) {
    ++skip;
  } else if( key == 'z' ) {
    num_points = max( 1, num_points - 1 );
    calcIncrement();
  } else if( key == 'x' ) {
    ++num_points;
    calcIncrement();
  } else if( key == 'g' ) {
    maker = new GifMaker( this, "output.gif" );
    maker.setRepeat( 0 );
    exp_frames = 0;
  }
}
